# -*- coding: utf-8 -*-
"""
Created on Fri May 26 09:13:56 2023

@author: heyunan
"""


import os
import numpy as np
import torch
from tqdm import tqdm
from scipy.integrate import simps
from ripser import ripser
from gtda.diagrams import BettiCurve
import pickle

def from_risper_dgm_to_gtda_dgm(risper_dgm):
    gtda_dgms = []
    for dim in [0, 1]:
        dim_col = np.full((len(risper_dgm[dim]), 1), dim)
        gtda_dgm = np.append(risper_dgm[dim],dim_col,axis=1)
        if dim == 0:
            gtda_dgm =  gtda_dgm[:-1, :]
        gtda_dgms.append(gtda_dgm)
    gtda_dgms = np.concatenate(gtda_dgms,axis=0)
    return gtda_dgms[None, :, :]


def compute_betti_curve(weight_path, dir_name):
    betti_curve = BettiCurve(n_bins=bins, n_jobs=-1)
    results_0, results_1 = [], []
    centroids_x_0, centroids_y_0 = [], []
    centroids_x_1, centroids_y_1 = [], []
    iterations = []

    files = os.listdir(weight_path)  # 获取文件夹中所有的文件名
    for file_name in tqdm(files, desc=f"Processing {dir_name}"):
        # 从文件名中获取迭代次数
        if "_params.pt" in file_name:
            iteration = int(file_name.split('_')[1])
            iterations.append(iteration)
            
            mnist_params = torch.load(os.path.join(weight_path, file_name))    
            mnist_weight = mnist_params["conv2.weight"].reshape(2048,9).cpu()
            risper_dgm = ripser(mnist_weight)['dgms']
            gtda_dgm = from_risper_dgm_to_gtda_dgm(risper_dgm)
            
            for betti_dim in [0, 1]:
                betti = betti_curve.fit_transform(gtda_dgm)
                x = betti_curve.samplings_[betti_dim]
                y = betti[0][betti_dim]  # 获取贝蒂-0曲线或贝蒂-1曲线

                # 计算面积的质心的x坐标和y坐标
                area = simps(y, x)
                x_c = simps(y * x, x) / area
                y_c = simps(y * y, x) / area

                if betti_dim == 0:
                    results_0.append((x, y))
                    centroids_x_0.append(x_c)
                    centroids_y_0.append(y_c)
                else:
                    results_1.append((x, y))
                    centroids_x_1.append(x_c)
                    centroids_y_1.append(y_c)
    
    return iterations, results_0, centroids_x_0, centroids_y_0, results_1, centroids_x_1, centroids_y_1

weights_dir = "./weights"
# dir_names = ['kaiming_uniform_run_1']
dir_names = [name for name in os.listdir(weights_dir) if os.path.isdir(os.path.join(weights_dir, name))]
print(dir_names)

bins = 10000

for d in dir_names:
    d_path = os.path.join(weights_dir, d)
    pickle_path = os.path.join(d_path, 'results.pickle')  # 创建一个新的pickle文件路径
    
    # 如果results.pickle文件已经存在，则跳过此次循环
    if os.path.exists(pickle_path):
        print(f"{pickle_path} already exists. Skipping computation for this directory.")
        continue
    # 计算贝蒂曲线和质心坐标
    iterations, betti_0_curves, betti_0_centroids_x, betti_0_centroids_y, betti_1_curves, betti_1_centroids_x, betti_1_centroids_y = compute_betti_curve(d_path, d)
  
    # 根据迭代次数排序所有数据
    data = list(zip(iterations, betti_0_curves, betti_0_centroids_x, betti_0_centroids_y, betti_1_curves, betti_1_centroids_x, betti_1_centroids_y))
    data.sort(key=lambda x: x[0])  # 按照第一个元素（迭代次数）进行排序

    # 保存结果到pickle文件
    with open(pickle_path, 'wb') as f:
        pickle.dump(data, f)

        
        